
#ifndef __PROTECT_H_
#define __PROTECT_H_

/**
 * @file protect.h
 *
 * Hlavickovy soubor uzivatelske casti rozhrani pro praci s ovladacem pproc.sys.
 * Dovoluje menit priznak chranenho procesu a testovat jeho stav. Tento soubor
 * obsahuje pouze definice "exportovanych" funkci.
 */

#include <windows.h>



// Popis vyznamu jednotlivych funkci naleznete u jejich implementace v souboru
// protect.c

BOOL ProtectProcess(HANDLE ProcessId);
BOOL UnprotectProcess(HANDLE ProcessId);
BOOL IsProtectedProcess(HANDLE ProcessId, PBOOL Result);

BOOL ProtectModuleInit(VOID);
VOID ProtectModuleFinit(VOID);


#endif
